import * as apigateway from 'aws-cdk-lib/aws-apigateway';
import * as lambda from 'aws-cdk-lib/aws-lambda';
import { Construct } from 'constructs';
import { SecretKey } from './secret-key';
/**
 * @experimental
 */
export interface RecaptchaAuthorizerProps {
    /**
     * (experimental) The minimum score threshold to allow by this authorizer.
     *
     * @default 0.5
     * @experimental
     */
    readonly scoreThreshold?: number;
    /**
     * (experimental) The actions to be allowed by this authorizer.
     *
     * @experimental
     */
    readonly allowedActions: string[];
    /**
     * (experimental) The reCaptcha API secret key.
     *
     * @experimental
     */
    readonly reCaptchaSecretKey: SecretKey;
    /**
     * (experimental) Enable AWS X-Ray Tracing for Lambda Function.
     *
     * @default Tracing.Disabled
     * @experimental
     */
    readonly tracing?: lambda.Tracing;
}
/**
 * (experimental) Request-based lambda authorizer that authorizes requests using Google's reCaptcha API.
 *
 * @experimental
 * @resource AWS::ApiGateway::Authorizer
 */
export declare class RecaptchaAuthorizer extends apigateway.Authorizer implements apigateway.IAuthorizer {
    private authorizer;
    /**
     * (experimental) The authorization type of this authorizer.
     *
     * @experimental
     */
    readonly authorizationType?: apigateway.AuthorizationType;
    /**
     * @experimental
     */
    constructor(scope: Construct, id: string, props: RecaptchaAuthorizerProps);
    /**
     * (experimental) The authorizer ID.
     *
     * @experimental
     * @attribute true
     */
    get authorizerId(): string;
    /**
     * Attaches this authorizer to a specific REST API.
     * @internal
     * @returns {void}
     * @param {RestApi} restApi the rest API to attach this authorizer to
     */
    _attachToApi(restApi: apigateway.RestApi): void;
}
