import * as iam from 'aws-cdk-lib/aws-iam';
import * as secretsmanager from 'aws-cdk-lib/aws-secretsmanager';
import * as ssm from 'aws-cdk-lib/aws-ssm';
/**
 * (experimental) A reCaptcha secret key.
 *
 * @experimental
 */
export declare abstract class SecretKey {
    /**
     * @param secretKey The secret key in plain text.
     * @returns a secret key from a string in plain text.
     * @experimental
     */
    static fromPlainText(secretKey: string): SecretKey;
    /**
     * @param secretKeyParameter The parameter in which the secret key is stored.
     * @returns a secret key from a parameter stored in AWS Systems Manager Parameter Store.
     * @experimental
     */
    static fromSsmParameter(secretKeyParameter: ssm.IParameter): SecretKey;
    /**
     * @param secretKeySecret The secret in which the secret key is stored.
     * @param field the name of the field with the value that you want to use as the secret key.
     * @returns a secret key from a secret stored in AWS Secrets Manager.
     * @experimental
     */
    static fromSecretsManager(secretKeySecret: secretsmanager.ISecret, field?: string): SecretKey;
    /**
     * (experimental) The type of secret key.
     *
     * @experimental
     */
    abstract readonly secretKeyType: string;
    /**
     * (experimental) Key-value pairs that should be added as environment variables to the Lambda.
     *
     * @experimental
     */
    abstract readonly environment: {
        [key: string]: string;
    };
    /**
     * (experimental) Grants reading the secret to a principal.
     *
     * @experimental
     */
    abstract grantRead?(grantee: iam.IGrantable): iam.Grant;
}
